@@ -6,6 +6,7 @@ use warnings;
 
 use Class::C3;
 use Class::Inspector;
+use Carp::Clan qw/^DBIx::Class/;
 
 sub inject_base {
   my ($class, $target, @to_inject) = @_;
@@ -64,13 +65,20 @@ sub _load_components {
 #
 # TODO: handle ->has_many('rel', 'Class'...) instead of
 #              ->has_many('rel', 'Some::Schema::Class'...)
+#
+# BUG: For some reason, packages with syntax errors are added to %INC on
+#      require
 sub ensure_class_loaded {
   my ($class, $f_class) = @_;
-  eval "require $f_class";
-  my $err = $@;
-  Class::Inspector->loaded($f_class)
-    or $class->throw_exception($err || "`require $f_class' was successful".
-                                       "but the package is not defined");
+  return if Class::Inspector->loaded($f_class);
+  eval "require $f_class"; # require needs a bareword or filename
+  if ($@) {
+    if ($class->can('throw_exception')) {
+      $class->throw_exception($@);
+    } else {
+      croak $@;
+    }
+  }
 }
 
 # Returns true if the specified class is installed or already loaded, false
