@@ -2,13 +2,13 @@ package DBIx::Class::InflateColumn;
 
 use strict;
 use warnings;
-
+use Scalar::Util qw/blessed/;
 
 use base qw/DBIx::Class::Row/;
 
 =head1 NAME
 
-DBIx::Class::InflateColumn - Automatically create objects from column data
+DBIx::Class::InflateColumn - Automatically create references from column data
 
 =head1 SYNOPSIS
 
@@ -20,13 +20,18 @@ DBIx::Class::InflateColumn - Automatically create objects from column data
 
 =head1 DESCRIPTION
 
-This component translates column data into objects, i.e. "inflating"
-the column data. It also "deflates" objects into an appropriate format
+This component translates column data into references, i.e. "inflating"
+the column data. It also "deflates" references into an appropriate format
 for the database.
 
 It can be used, for example, to automatically convert to and from
 L<DateTime> objects for your date and time fields.
 
+It will accept arrayrefs, hashrefs and blessed references (objects),
+but not scalarrefs. Scalar references are passed through to the
+database to deal with, to allow such settings as C< \'year + 1'> and
+C< \'DEFAULT' > to work.
+
 =head1 METHODS
 
 =head2 inflate_column
@@ -85,7 +90,9 @@ sub _inflated_column {
 
 sub _deflated_column {
   my ($self, $col, $value) = @_;
-  return $value unless ref $value; # If it's not an object, don't touch it
+#  return $value unless ref $value && blessed($value); # If it's not an object, don't touch it
+  ## Leave scalar refs (ala SQL::Abstract literal SQL), untouched, deflate all other refs
+  return $value unless (ref $value && ref($value) ne 'SCALAR');
   my $info = $self->column_info($col) or
     $self->throw_exception("No column info for $col");
   return $value unless exists $info->{_inflate_info};
@@ -109,7 +116,6 @@ sub get_inflated_column {
   my ($self, $col) = @_;
   $self->throw_exception("$col is not an inflated column")
     unless exists $self->column_info($col)->{_inflate_info};
-
   return $self->{_inflated_column}{$col}
     if exists $self->{_inflated_column}{$col};
   return $self->{_inflated_column}{$col} =
@@ -126,9 +132,15 @@ analogous to L<DBIx::Class::Row/set_column>.
 =cut
 
 sub set_inflated_column {
-  my ($self, $col, @rest) = @_;
-  my $ret = $self->_inflated_column_op('set', $col, @rest);
-  return $ret;
+  my ($self, $col, $inflated) = @_;
+  $self->set_column($col, $self->_deflated_column($col, $inflated));
+#  if (blessed $inflated) {
+  if (ref $inflated && ref($inflated) ne 'SCALAR') {
+    $self->{_inflated_column}{$col} = $inflated; 
+  } else {
+    delete $self->{_inflated_column}{$col};      
+  }
+  return $inflated;
 }
 
 =head2 store_inflated_column
@@ -136,70 +148,20 @@ sub set_inflated_column {
   my $copy = $obj->store_inflated_column($col => $val);
 
 Sets a column value from an inflated value without marking the column
-as dirty.  This is directly analogous to
-L<DBIx::Class::Row/store_column>.
+as dirty. This is directly analogous to L<DBIx::Class::Row/store_column>.
 
 =cut
 
 sub store_inflated_column {
-  my ($self, $col, @rest) = @_;
-  my $ret = $self->_inflated_column_op('store', $col, @rest);
-  return $ret;
-}
-
-sub _inflated_column_op {
-  my ($self, $op, $col, $obj) = @_;
-  my $meth = "${op}_column";
-  unless (ref $obj) {
-    delete $self->{_inflated_column}{$col};
-    return $self->$meth($col, $obj);
+  my ($self, $col, $inflated) = @_;
+#  unless (blessed $inflated) {
+  unless (ref $inflated && ref($inflated) ne 'SCALAR') {
+      delete $self->{_inflated_column}{$col};
+      $self->store_column($col => $inflated);
+      return $inflated;
   }
-
-  my $deflated = $self->_deflated_column($col, $obj);
-           # Do this now so we don't store if it's invalid
-
-  $self->{_inflated_column}{$col} = $obj;
-  $self->$meth($col, $deflated);
-  return $obj;
-}
-
-=head2 update
-
-Updates a row in the same way as L<DBIx::Class::Row/update>, handling
-inflation and deflation of columns appropriately.
-
-=cut
-
-sub update {
-  my ($class, $attrs, @rest) = @_;
-  $attrs ||= {};
-  foreach my $key (keys %$attrs) {
-    if (ref $attrs->{$key}
-          && exists $class->column_info($key)->{_inflate_info}) {
-#      $attrs->{$key} = $class->_deflated_column($key, $attrs->{$key});
-      $class->set_inflated_column ($key, delete $attrs->{$key});
-    }
-  }
-  return $class->next::method($attrs, @rest);
-}
-
-=head2 new
-
-Creates a row in the same way as L<DBIx::Class::Row/new>, handling
-inflation and deflation of columns appropriately.
-
-=cut
-
-sub new {
-  my ($class, $attrs, @rest) = @_;
-  $attrs ||= {};
-  foreach my $key (keys %$attrs) {
-    if (ref $attrs->{$key}
-          && exists $class->column_info($key)->{_inflate_info}) {
-      $attrs->{$key} = $class->_deflated_column($key, $attrs->{$key});
-    }
-  }
-  return $class->next::method($attrs, @rest);
+  delete $self->{_column_data}{$col};
+  return $self->{_inflated_column}{$col} = $inflated;
 }
 
 =head1 SEE ALSO
@@ -220,6 +182,8 @@ Matt S. Trout <mst@shadowcatsystems.co.uk>
 
 Daniel Westermann-Clark <danieltwc@cpan.org> (documentation)
 
+Jess Robinson <cpan@desert-island.demon.co.uk>
+
 =head1 LICENSE
 
 You may distribute this code under the same terms as Perl itself.
